# @file globals.py
# @func module which starts the session with Nao robot. 
# @auth Hessel van der Molen
# @date 4 May 2012
# @update 2023 by Arnoud Visser
# v7 Jan 19, 2023 - defines two new functions startSession and StopSession
# v7 Jan 19, 2023 - includes sysProxy to check versions in main

from naoqi import qi
import sys

class globals_v8f:
    ipadress = "146.50.60.39" # Phineas  
    ipadress = "146.50.60.35" # Ferdinand
    port = "9559"

    def setDependencies(self, modules):
        
        print ("Configured to connect to Naoqi at ip \"" + self.ipadress + "\" on port " + self.port +".\n")
        

    def startSession(self):
        # Initialize qi framework.
        connection_url = "tcp://" + self.ipadress + ":" + self.port
        try:
             self.app = qi.Application(["BehaviorBasedRobotics",
                          "--qi-url=" + connection_url])
        except RuntimeError:
             print("After an error the session is still running in the background.")
             print("Restart the Spyder-kernel by using 'Ctr+.' or using the Consoles option at Spyder-menu.")
             sys.exit(1)
             #qi._app = None # resetting
             
             #self.session.disconnect()
             #self.app.stop()
             
        try:
             self.app.start()
        except RuntimeError:
             print ("Can't connect to Naoqi at ip \"" + self.ipadress + "\" on port " + self.port +".\n")
             sys.exit(2)

        self.session = self.app.session
        
    def setProxies(self):
        self.motProxy = self.app.session.service("ALMotion")
        self.lifeProxy = self.app.session.service("ALAutonomousLife")
        self.posProxy = self.app.session.service("ALRobotPosture")
        self.vidProxy = self.app.session.service("ALVideoDevice")	
        self.sysProxy = self.session.service("ALSystem")
        
        # for Ferdinand, to remove the warnings

        self.diagnosisProxy = self.session.service("ALDiagnosis")
        self.notificationProxy = self.session.service("ALNotificationManager")
	
	
        # Subscribe to sonars, this will launch sonars (at hardware level) and start data acquisition.
        

        #Uncomment to access sonars
        self.sonarProxy = self.session.service("ALSonar")
        #self.memoryProxy = self.session.service("ALMemory")
        
    def stopSession(self):
        try:
            print("Trying to stop the session")
            #self.app.session.disconnect()
            self.app.stop()
            print("The connection with the Nao is stopped")
        except RuntimeError:
            print("Failed to stop session")    
		
    def unsetDependencies(self, modules):
        print ("Nothing to do for the global dependencies") 
        
